/*
 *  TOPPERS Software
 *      Toyohashi Open Platform for Embedded Real-Time Systems
 *
 *  Copyright (C) 2006-2016 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2018,2019 by Naoki Saito
 *             Nagoya Municipal Industrial Research Institute, JAPAN
 *
 *  上記著作権者は，以下の(1)〜(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 *
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 *
 */

/*
 *  チップ依存処理(BCM2837用)
 *
 *  このヘッダファイルは，target_config.h（または，そこからインクルード
 *  されるファイル）のみからインクルードされる．
 *  他のファイルから直接インクルードしてはならない．
 */

#ifndef TOPPERS_CHIP_CONFIG_H
#define TOPPERS_CHIP_CONFIG_H

/*
 *  ハードウェア定義
 */
#include "bcm2837.h"

/*
 *  割込み関連
 */

/*
 *  割込み優先度の変換
 */
#define EXT_IPM(pri)  (-pri)    // 内部表現から外部表現へ
#define INT_IPM(ipm)  (-ipm)    // 外部表現から内部表現へ

/*
 *  TIPM_ENAALL（割込み優先度マスク全解除）の内部表現
 */
#define IIPM_ENAALL  (0)

/*
 *  割込み番号の範囲の判定
 */
#define VALID_INTNO(intno)           ((TMIN_INTNO <= (intno)) && ((intno) <= TMAX_INTNO))
#define VALID_INTNO_DISINT(intno)    VALID_INTNO(intno)

/*
 *  割込みハンドラの出入口処理の生成マクロ
 */
#define INT_ENTRY(inhno, inthdr)    inthdr
#define INTHDR_ENTRY(inhno, inhno_num, inthdr) extern void inthdr(void);

#ifndef TOPPERS_MACRO_ONLY

/*
 *  CPUロック状態への移行
 */
Inline void
x_lock_cpu(void)
{
    aarch64_set_daif_i();
}

#define t_lock_cpu()    x_lock_cpu()
#define i_lock_cpu()    x_lock_cpu()

/*
 *  CPUロック状態の解除
 */
Inline void
x_unlock_cpu(void)
{
    aarch64_clear_daif_i();
}

#define t_unlock_cpu()    x_unlock_cpu()
#define i_unlock_cpu()    x_unlock_cpu()

/*
 *  CPUロック状態の参照
 */
Inline bool_t
x_sense_lock(void)
{
    bool_t state;

    if(aarch64_sense_daif_irq()) {
        state = true;
    }
    else {
        state = false;
    }

    return state;
}

#define t_sense_lock()    x_sense_lock()
#define i_sense_lock()    x_sense_lock()

/*
 *  プロセッサの割込み優先度
 *    チップの割込みコントローラには割込み優先度による制御機能が
 *    実装されていないため，割込み優先度を格納する変数を用意する
 */
extern PRI current_intpri;

/*
 *  割込みハンドラの設定
 */
Inline void
x_define_inh(INHNO inhno, FP int_entry)
{
}

/*
 *  割込みハンドラ入口で必要なIRC操作
 */
Inline void
i_begin_int(INTNO intno)
{
}

/*
 *  割込みハンドラの出口で必要なIRC操作
 */
Inline void
i_end_int(INTNO intno)
{
}

/*
 *  割込み要求ラインの属性の設定
 */
extern void x_config_int(INTNO intno, ATR intatr, PRI intpri);

/*
 *  割込み優先度にもとづく割込みのマスク
 *    割込みの入口で，受け付けた割込みの割込みレベルに応じて
 *    割込みをマスクする
 */
extern void chip_mask_interrupt(PRI pri);

/*
 *  割込みマスクの解除
 *    割込みの出口で，受け付け直前の割込みレベルをもとに
 *    割込みのマスクを解除する
 */
extern void chip_unmask_interrupt(PRI pri);

/*
 *  割込み入口の割込み操作（chip_support.S）
 */
extern void chip_int_entry(void);

/*
 *  割込み優先度テーブル(kernel_cfg.c)
 */
extern const PRI intpri_table[];

/*
 *  割込みマスクテーブル(kernel_cfg.c)
 */
extern const uint32_t iipm_mask_table[];

/*
 *  割込みハンドラテーブル(kernel_cfg.c)
 */
extern const INTHDR inh_table[];

#endif /* TOPPERS_MACRO_ONLY */



/*
 *  CPU例外関連
 */

/*
 *  CPU例外ハンドラ番号
 */
#define VALID_EXCNO_DEFEXC(excno)    (TMIN_EXCNO <= (excno) && (excno) <= TMAX_EXCNO)

/*
 *  CPU例外ハンドラの入口処理の生成マクロ
 */
#define EXC_ENTRY(excno, exchdr)    exchdr
#define EXCHDR_ENTRY(excno, excno_num, exchdr) extern void exchdr(void *p_excinf);


#ifndef TOPPERS_MACRO_ONLY

/*
 *  CPU例外ハンドラの設定
 */
Inline void
x_define_exc(EXCNO excno, FP exc_entry)
{
}

/*
 *  例外入口の割込み操作（chip_support.S）
 */
extern void chip_exc_entry(void);

/*
 *  例外ハンドラテーブル(kernel_cfg.c)
 */
extern const EXCHDR exc_table[];

#endif /* TOPPERS_MACRO_ONLY */

/*
 * その他
 */

#ifndef TOPPERS_MACRO_ONLY

/*
 *  チップ依存の初期化
 */
extern void chip_initialize(void);

/*
 *  チップ依存の終了処理
 */
extern void chip_terminate(void) NoReturn;

#endif /* TOPPERS_MACRO_ONLY */

/*
 *  コア依存定義
 */
#include "arm64_gcc/common/core_config.h"

#endif /* TOPPERS_CHIP_CONFIG_H */
